=== modified file 'client/wakeup.h'
--- client/wakeup.h	2010-11-01 06:01:49 +0000
+++ client/wakeup.h	2010-11-17 21:25:31 +0000
@@ -20,6 +20,7 @@
 
 #include <boost/thread/mutex.hpp>
 #include <boost/thread/condition_variable.hpp>
+#include <boost/shared_ptr.hpp>
 
 #ifndef CLIENT_WAKEUP_H
 #define CLIENT_WAKEUP_H
@@ -28,15 +29,14 @@
 
 // Wakeup starts in a blocking posistion
 class Wakeup {
-  bool sleeping;
-  boost::mutex sleeper_mutex;
-  boost::condition_variable_any sleep_threshhold;
-
 public:
+  typedef boost::shared_ptr<Wakeup> shared_ptr;
+
   Wakeup() :
     sleeping(true)
   { }
 
+  // Signal all of the waiters to start
   void start()
   {
     boost::mutex::scoped_lock scopedWakeup(sleeper_mutex);
@@ -44,6 +44,7 @@
     sleep_threshhold.notify_all();
   }
 
+  // reset after the start of the signal so that we can reuse the wakeup
   void reset()
   {
     boost::mutex::scoped_lock scopedWakeup(sleeper_mutex);
@@ -59,6 +60,11 @@
     }
     sleeper_mutex.unlock();
   }
+
+private:
+  bool sleeping;
+  boost::mutex sleeper_mutex;
+  boost::condition_variable_any sleep_threshhold;
 };
 
 } // namespace client

=== modified file 'drizzled/enum.h'
--- drizzled/enum.h	2010-10-31 21:54:29 +0000
+++ drizzled/enum.h	2010-11-17 21:25:31 +0000
@@ -1048,7 +1048,11 @@
   ER_OUT_OF_GLOBAL_JOINMEMORY,
   ER_OUT_OF_GLOBAL_READRNDMEMORY,
   ER_OUT_OF_GLOBAL_READMEMORY,
-  ER_ERROR_LAST= ER_OUT_OF_GLOBAL_READMEMORY
+  ER_USER_LOCKS_CANT_WAIT_ON_OWN_BARRIER,
+  ER_USER_LOCKS_UNKNOWN_BARRIER,
+  ER_USER_LOCKS_NOT_OWNER_OF_BARRIER,
+
+  ER_ERROR_LAST= ER_USER_LOCKS_NOT_OWNER_OF_BARRIER
 };
 
 enum drizzle_exit_codes {

=== modified file 'drizzled/error.cc'
--- drizzled/error.cc	2010-10-31 21:54:29 +0000
+++ drizzled/error.cc	2010-11-17 21:25:31 +0000
@@ -479,6 +479,11 @@
   add(ER_UNKNOWN_ENGINE_OPTION, N_("Unknown table engine option key/pair %s = %s."));
   add(ER_UNKNOWN_SCHEMA_OPTION, N_("Unknown schema engine option key/pair %s = %s."));
 
+  // User lock/barrier error messages
+  add(ER_USER_LOCKS_CANT_WAIT_ON_OWN_BARRIER, N_("wait() can not be called on session owning barrier."));
+  add(ER_USER_LOCKS_UNKNOWN_BARRIER, N_("Unknown barrier requested."));
+  add(ER_USER_LOCKS_NOT_OWNER_OF_BARRIER, N_("Session does not own barrier."));
+
   add(EE_CANTUNLOCK, N_("Can't unlock file (Errcode: %d)"));
   add(EE_CANT_CHSIZE, N_("Can't change size of file (Errcode: %d)"));
   add(EE_CANT_OPEN_STREAM, N_("Can't open stream from handle (Errcode: %d)"));

=== added file 'plugin/user_locks/barrier_dictionary.cc'
--- plugin/user_locks/barrier_dictionary.cc	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/barrier_dictionary.cc	2010-11-17 21:25:31 +0000
@@ -0,0 +1,69 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+
+#include "plugin/user_locks/module.h"
+
+#include <drizzled/atomics.h>
+#include <drizzled/session.h>
+
+using namespace drizzled;
+using namespace std;
+
+user_locks::UserBarriers::UserBarriers() :
+  plugin::TableFunction("DATA_DICTIONARY", "USER_BARRIERS")
+{
+  add_field("USER_BARRIERS", plugin::TableFunction::STRING, user_locks::LARGEST_LOCK_NAME, false);
+#if 0
+  add_field("SESSION_ID", plugin::TableFunction::NUMBER, 0, false);
+  add_field("USER_NAME", plugin::TableFunction::STRING);
+#endif
+}
+
+user_locks::UserBarriers::Generator::Generator(drizzled::Field **arg) :
+  drizzled::plugin::TableFunction::Generator(arg)
+{
+  user_locks::Barriers::getInstance().Copy(barrier_map);
+  iter= barrier_map.begin();
+}
+
+bool user_locks::UserBarriers::Generator::populate()
+{
+
+  while (iter != barrier_map.end())
+  {
+    // USER_LOCK_NAME
+    push((*iter).first.getLockName());
+
+#if 0
+    // SESSION_ID
+    push((*iter).second->id);
+    //
+    // USER_NAME
+    push((*iter).first.getUser());
+#endif
+
+    iter++;
+    return true;
+  }
+
+  return false;
+}

=== added file 'plugin/user_locks/barrier_dictionary.h'
--- plugin/user_locks/barrier_dictionary.h	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/barrier_dictionary.h	2010-11-17 21:25:31 +0000
@@ -0,0 +1,51 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_USER_LOCKS_BARRIER_DICTIONARY_H
+#define PLUGIN_USER_LOCKS_BARRIER_DICTIONARY_H
+
+namespace user_locks {
+
+class UserBarriers : public  drizzled::plugin::TableFunction
+{
+
+public:
+  UserBarriers();
+
+  class Generator : public drizzled::plugin::TableFunction::Generator 
+  {
+    Barriers::Map barrier_map;
+    Barriers::Map::iterator iter;
+
+  public:
+    Generator(drizzled::Field **arg);
+
+    bool populate();
+  };
+
+  Generator *generator(drizzled::Field **arg)
+  {
+    return new Generator(arg);
+  }
+};
+
+} /* namespace user_locks */
+
+#endif /* PLUGIN_USER_LOCKS_BARRIER_DICTIONARY_H */

=== added file 'plugin/user_locks/barrier_storage.h'
--- plugin/user_locks/barrier_storage.h	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/barrier_storage.h	2010-11-17 21:25:31 +0000
@@ -0,0 +1,84 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_USER_LOCKS_BARRIER_STORAGE_H
+#define PLUGIN_USER_LOCKS_BARRIER_STORAGE_H
+
+#include "drizzled/session.h"
+
+#include <boost/unordered_set.hpp>
+
+namespace user_locks {
+namespace barriers {
+
+/*
+  We a storable to track any locks we might have open so that if we are disconnected before we release the locks, we release the locks during the deconstruction of Session.
+*/
+
+const std::string property_key("user_barriers");
+
+class Storable : public drizzled::util::Storable {
+  Keys list_of_locks;
+  drizzled::session_id_t id;
+
+public:
+
+  Storable(drizzled::session_id_t id_arg) :
+    id(id_arg)
+  {
+  }
+
+  ~Storable()
+  {
+    erase_all();
+  }
+
+  void insert(const Key &arg)
+  {
+    list_of_locks.insert(arg);
+  }
+
+  bool erase(const Key &arg)
+  {
+    return boost::lexical_cast<bool>(list_of_locks.erase(arg));
+  }
+
+  // An assert() should be added so that we can test to make sure the locks
+  // are what we think they are (i.e. test the result of release())
+  int64_t erase_all()
+  {
+    int64_t count= list_of_locks.size();
+
+    for (Keys::iterator iter= list_of_locks.begin();
+         iter != list_of_locks.end(); iter++)
+    {
+      (void)user_locks::Barriers::getInstance().release(*iter);
+    }
+    list_of_locks.clear();
+
+    return count;
+  }
+};
+
+
+} /* namespace barriers */
+} /* namespace user_locks */
+
+#endif /* PLUGIN_USER_LOCKS_BARRIER_STORAGE_H */

=== added file 'plugin/user_locks/barriers.cc'
--- plugin/user_locks/barriers.cc	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/barriers.cc	2010-11-17 21:25:31 +0000
@@ -0,0 +1,71 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+#include "plugin/user_locks/module.h"
+
+#include <boost/thread/locks.hpp>
+
+#include <string>
+
+namespace user_locks {
+
+bool Barriers::create(const user_locks::Key &arg)
+{
+  boost::unique_lock<boost::mutex> scope(mutex);
+  return barrier_map.insert(std::make_pair(arg, new client::Wakeup())).second;
+}
+
+boost::tribool Barriers::release(const user_locks::Key &arg)
+{
+  size_t elements= 0;
+  boost::unique_lock<boost::mutex> scope(mutex);
+  Map::iterator iter= barrier_map.find(arg);
+
+  // Nothing is found
+  if ( iter == barrier_map.end())
+    return boost::indeterminate;
+
+  (*iter).second->start(); // We tell anyone left to start running
+  elements= barrier_map.erase(arg);
+
+  if (elements)
+    return true;
+
+  return false;
+}
+
+client::Wakeup::shared_ptr Barriers::find(const user_locks::Key &arg)
+{
+  boost::unique_lock<boost::mutex> scope(mutex);
+  Map::iterator iter= barrier_map.find(arg);
+
+  if (iter != barrier_map.end())
+    return (*iter).second;
+
+  return client::Wakeup::shared_ptr();
+}
+
+void Barriers::Copy(Map &arg)
+{
+  arg= barrier_map;
+}
+
+} /* namespace user_locks */

=== added file 'plugin/user_locks/barriers.h'
--- plugin/user_locks/barriers.h	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/barriers.h	2010-11-17 21:25:31 +0000
@@ -0,0 +1,67 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <boost/thread/mutex.hpp>
+#include <boost/thread/condition_variable.hpp>
+#include <boost/unordered_map.hpp>
+#include <boost/logic/tribool.hpp>
+#include <boost/unordered/unordered_set.hpp>
+
+#include <string>
+
+#include "drizzled/session.h"
+#include "drizzled/util/string.h"
+
+
+#ifndef PLUGIN_USER_LOCKS_BARRIERS_H
+#define PLUGIN_USER_LOCKS_BARRIERS_H
+
+#include "client/wakeup.h"
+
+namespace user_locks {
+
+const size_t LARGEST_BARRIER_NAME= 64;
+
+class Barriers
+{
+public:
+  typedef boost::unordered_map<user_locks::Key, client::Wakeup::shared_ptr> Map;
+
+  static Barriers &getInstance(void)
+  {
+    static Barriers instance;
+    return instance;
+  }
+
+  bool create(const user_locks::Key &arg);
+  boost::tribool release(const user_locks::Key &arg);
+  client::Wakeup::shared_ptr find(const user_locks::Key &arg);
+  void Copy(Map &arg);
+
+private:
+
+  boost::mutex mutex;
+  Map barrier_map; 
+};
+
+
+} /* namespace user_locks */
+
+#endif /* PLUGIN_USER_LOCKS_BARRIERS_H */

=== added file 'plugin/user_locks/create_barrier.cc'
--- plugin/user_locks/create_barrier.cc	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/create_barrier.cc	2010-11-17 21:25:31 +0000
@@ -0,0 +1,72 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+#include "plugin/user_locks/module.h"
+#include "plugin/user_locks/barrier_storage.h"
+
+#include <string>
+
+namespace user_locks {
+
+int64_t CreateBarrier::val_int()
+{
+  drizzled::String *res= args[0]->val_str(&value);
+  int64_t wait_time= 0;
+
+  if (arg_count == 2)
+  {
+    wait_time= args[1]->val_int();
+  }
+
+  if (not res)
+  {
+    null_value= true;
+    return 0;
+  }
+  null_value= false;
+
+  if (not res->length())
+    return 0;
+
+  barriers::Storable *list= static_cast<barriers::Storable *>(getSession().getProperty(barriers::property_key));
+
+  boost::tribool result= user_locks::Barriers::getInstance().create(Key(getSession().getSecurityContext(), res->c_str()));
+
+  if (boost::indeterminate(result))
+    null_value= true;
+
+  if (result)
+  {
+    if (not list)
+    {
+      list= new barriers::Storable(getSession().getSessionId());
+      getSession().setProperty(barriers::property_key, list);
+    }
+
+    list->insert(Key(getSession().getSecurityContext(), res->c_str()));
+
+    return 1;
+  }
+
+  return 0;
+}
+
+} /* namespace user_locks */

=== added file 'plugin/user_locks/create_barrier.h'
--- plugin/user_locks/create_barrier.h	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/create_barrier.h	2010-11-17 21:25:31 +0000
@@ -0,0 +1,42 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_USER_LOCKS_CREATE_BARRIER_H
+#define PLUGIN_USER_LOCKS_CREATE_BARRIER_H
+
+namespace user_locks {
+
+class CreateBarrier : public drizzled::Item_int_func
+{
+  drizzled::String value;
+
+public:
+  CreateBarrier() :
+    drizzled::Item_int_func()
+  {}
+
+  int64_t val_int();
+  const char *func_name() const { return "create_barrier"; }
+  bool check_argument_count(int n) { return n < 3; }
+};
+
+} /* namespace user_locks */
+
+#endif /* PLUGIN_USER_LOCKS_CREATE_BARRIER_H */

=== modified file 'plugin/user_locks/locks.cc'
--- plugin/user_locks/locks.cc	2010-10-30 01:19:00 +0000
+++ plugin/user_locks/locks.cc	2010-11-17 21:25:31 +0000
@@ -109,6 +109,7 @@
 
 void Locks::Copy(LockMap &lock_map_arg)
 {
+  //@todo add lock(?)
   lock_map_arg= lock_map;
 }
 

=== modified file 'plugin/user_locks/module.cc'
--- plugin/user_locks/module.cc	2010-10-12 21:41:44 +0000
+++ plugin/user_locks/module.cc	2010-11-17 21:25:31 +0000
@@ -25,7 +25,15 @@
 
 static int init(drizzled::module::Context &context)
 {
-  user_locks::Locks::getInstance(); // We are single threaded at this point, so we can use this to initialize.
+  // We are single threaded at this point, so we can use this to initialize.
+  user_locks::Locks::getInstance();
+  user_locks::Barriers::getInstance();
+
+  context.add(new plugin::Create_function<user_locks::CreateBarrier>("create_barrier"));
+  context.add(new plugin::Create_function<user_locks::barriers::Release>("release_barrier"));
+  context.add(new plugin::Create_function<user_locks::barriers::Wait>("wait"));
+  context.add(new plugin::Create_function<user_locks::barriers::Signal>("signal"));
+
   context.add(new plugin::Create_function<user_locks::GetLock>("get_lock"));
   context.add(new plugin::Create_function<user_locks::GetLocks>("get_locks"));
   context.add(new plugin::Create_function<user_locks::ReleaseLock>("release_lock"));
@@ -33,6 +41,7 @@
   context.add(new plugin::Create_function<user_locks::IsFreeLock>("is_free_lock"));
   context.add(new plugin::Create_function<user_locks::IsUsedLock>("is_used_lock"));
   context.add(new user_locks::UserLocks);
+  context.add(new user_locks::UserBarriers);
 
   return 0;
 }

=== modified file 'plugin/user_locks/module.h'
--- plugin/user_locks/module.h	2010-10-13 02:49:48 +0000
+++ plugin/user_locks/module.h	2010-11-17 21:25:31 +0000
@@ -30,11 +30,18 @@
 // We use this key for all of the locks we create.
 #include "plugin/user_locks/key.h"
 
+#include "plugin/user_locks/create_barrier.h"
+#include "plugin/user_locks/barriers.h"
+#include "plugin/user_locks/barrier_dictionary.h"
+#include "plugin/user_locks/release_barrier.h"
+#include "plugin/user_locks/wait.h"
+#include "plugin/user_locks/signal.h"
 #include "plugin/user_locks/get_lock.h"
 #include "plugin/user_locks/get_locks.h"
 #include "plugin/user_locks/is_free_lock.h"
 #include "plugin/user_locks/is_used_lock.h"
 #include "plugin/user_locks/locks.h"
+#include "plugin/user_locks/barriers.h"
 #include "plugin/user_locks/release_lock.h"
 #include "plugin/user_locks/release_locks.h"
 #include "plugin/user_locks/user_locks_dictionary.h"

=== modified file 'plugin/user_locks/plugin.ini'
--- plugin/user_locks/plugin.ini	2010-10-13 02:49:48 +0000
+++ plugin/user_locks/plugin.ini	2010-11-17 21:25:31 +0000
@@ -3,6 +3,9 @@
 description=Provides lock functions to users.
 load_by_default=yes
 sources= 
+  barrier_dictionary.cc
+  barriers.cc
+  create_barrier.cc
   get_lock.cc
   get_locks.cc
   is_free_lock.cc
@@ -10,10 +13,17 @@
   key.cc
   locks.cc
   module.cc
+  release_barrier.cc
   release_lock.cc
   release_locks.cc
+  signal.cc
   user_locks_dictionary.cc
+  wait.cc
 headers=
+  barrier_dictionary.h
+  barrier_storage.h
+  barriers.h
+  create_barrier.h
   get_lock.h
   get_locks.h
   is_free_lock.h
@@ -22,7 +32,10 @@
   lock_storage.h
   locks.h
   module.h
+  release_barrier.h
   release_lock.h
   release_locks.h
+  signal.h
   user_locks_dictionary.h
+  wait.h
 static=yes

=== added file 'plugin/user_locks/release_barrier.cc'
--- plugin/user_locks/release_barrier.cc	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/release_barrier.cc	2010-11-17 21:25:31 +0000
@@ -0,0 +1,66 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+#include "plugin/user_locks/module.h"
+#include "plugin/user_locks/barrier_storage.h"
+
+#include <string>
+
+namespace user_locks {
+namespace barriers {
+
+int64_t Release::val_int()
+{
+  drizzled::String *res= args[0]->val_str(&value);
+
+  if (not res)
+  {
+    null_value= true;
+    return 0;
+  }
+  null_value= false;
+
+  if (not res->length())
+    return 0;
+
+  boost::tribool result= Barriers::getInstance().release(Key(getSession().getSecurityContext(), res->c_str()));
+
+  if (result)
+  {
+    Storable *list= static_cast<Storable *>(getSession().getProperty(property_key));
+    if (list) // We may have not been the owner of the barrier
+      list->erase(Key(getSession().getSecurityContext(), res->c_str()));
+
+    return 1;
+  }
+  else if (not result)
+  {
+    my_error(drizzled::ER_USER_LOCKS_UNKNOWN_BARRIER, MYF(0));
+    return 0;
+  }
+
+  null_value= true;
+
+  return 0;
+}
+
+} /* namespace barriers */
+} /* namespace user_locks */

=== added file 'plugin/user_locks/release_barrier.h'
--- plugin/user_locks/release_barrier.h	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/release_barrier.h	2010-11-17 21:25:31 +0000
@@ -0,0 +1,44 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_USER_LOCKS_RELEASE_BARRIER_H
+#define PLUGIN_USER_LOCKS_RELEASE_BARRIER_H
+
+namespace user_locks {
+namespace barriers {
+
+class Release : public drizzled::Item_int_func
+{
+  drizzled::String value;
+
+public:
+  Release() :
+    drizzled::Item_int_func()
+  {}
+
+  int64_t val_int();
+  const char *func_name() const { return "release_barrier"; }
+  bool check_argument_count(int n) { return n == 1; }
+};
+
+} /* namespace barriers */
+} /* namespace user_locks */
+
+#endif /* PLUGIN_USER_LOCKS_RELEASE_BARRIER_H */

=== added file 'plugin/user_locks/signal.cc'
--- plugin/user_locks/signal.cc	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/signal.cc	2010-11-17 21:25:31 +0000
@@ -0,0 +1,59 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+#include "plugin/user_locks/module.h"
+#include "plugin/user_locks/barrier_storage.h"
+
+#include <string>
+
+namespace user_locks {
+namespace barriers {
+
+int64_t Signal::val_int()
+{
+  drizzled::String *res= args[0]->val_str(&value);
+
+  if (not res)
+  {
+    null_value= true;
+    return 0;
+  }
+  null_value= false;
+
+  if (not res->length())
+    return 0;
+
+  client::Wakeup::shared_ptr barrier= Barriers::getInstance().find(Key(getSession().getSecurityContext(), res->c_str()));
+
+  if (barrier)
+  {
+    barrier->start();
+
+    return 1;
+  }
+
+  null_value= true;
+
+  return 0;
+}
+
+} /* namespace barriers */
+} /* namespace user_locks */

=== added file 'plugin/user_locks/signal.h'
--- plugin/user_locks/signal.h	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/signal.h	2010-11-17 21:25:31 +0000
@@ -0,0 +1,44 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_USER_LOCKS_SIGNAL_H
+#define PLUGIN_USER_LOCKS_SIGNAL_H
+
+namespace user_locks {
+namespace barriers {
+
+class Signal : public drizzled::Item_int_func
+{
+  drizzled::String value;
+
+public:
+  Signal() :
+    drizzled::Item_int_func()
+  {}
+
+  int64_t val_int();
+  const char *func_name() const { return "signal"; }
+  bool check_argument_count(int n) { return n == 1; }
+};
+
+} /* namespace barriers */
+} /* namespace user_locks */
+
+#endif /* PLUGIN_USER_LOCKS_SIGNAL_H */

=== added file 'plugin/user_locks/tests/r/create_barrier.result'
--- plugin/user_locks/tests/r/create_barrier.result	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/tests/r/create_barrier.result	2010-11-17 21:25:31 +0000
@@ -0,0 +1,35 @@
+SELECT create_barrier("foo");
+create_barrier("foo")
+1
+SELECT * FROM DATA_DICTIONARY.USER_BARRIERS;
+USER_BARRIERS
+foo
+SELECT release_barrier("foo");
+release_barrier("foo")
+1
+SELECT * FROM DATA_DICTIONARY.USER_BARRIERS;
+USER_BARRIERS
+SELECT create_barrier("foo1");
+create_barrier("foo1")
+1
+SELECT create_barrier("foo2");
+create_barrier("foo2")
+1
+SELECT * FROM DATA_DICTIONARY.USER_BARRIERS;
+USER_BARRIERS
+foo1
+foo2
+SELECT release_barrier("foo2");
+release_barrier("foo2")
+1
+SELECT * FROM DATA_DICTIONARY.USER_BARRIERS;
+USER_BARRIERS
+foo1
+SELECT release_barrier("foo1");
+release_barrier("foo1")
+1
+SELECT * FROM DATA_DICTIONARY.USER_BARRIERS;
+USER_BARRIERS
+SELECT release_barrier("foo2");
+release_barrier("foo2")
+NULL

=== added file 'plugin/user_locks/tests/r/dangling_barrier.result'
--- plugin/user_locks/tests/r/dangling_barrier.result	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/tests/r/dangling_barrier.result	2010-11-17 21:25:31 +0000
@@ -0,0 +1,4 @@
+set @lock_name= "dangling_barrier";
+SELECT get_lock(@lock_name);
+get_lock(@lock_name)
+1

=== added file 'plugin/user_locks/tests/t/create_barrier.test'
--- plugin/user_locks/tests/t/create_barrier.test	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/tests/t/create_barrier.test	2010-11-17 21:25:31 +0000
@@ -0,0 +1,20 @@
+SELECT create_barrier("foo");
+SELECT * FROM DATA_DICTIONARY.USER_BARRIERS;
+SELECT release_barrier("foo");
+SELECT * FROM DATA_DICTIONARY.USER_BARRIERS;
+
+SELECT create_barrier("foo1");
+SELECT create_barrier("foo2");
+SELECT * FROM DATA_DICTIONARY.USER_BARRIERS;
+SELECT release_barrier("foo2");
+SELECT * FROM DATA_DICTIONARY.USER_BARRIERS;
+SELECT release_barrier("foo1");
+SELECT * FROM DATA_DICTIONARY.USER_BARRIERS;
+
+# This should complain about a non-existant barrier
+SELECT release_barrier("foo2");
+
+# foo2 does not exist, so we should not wait for it
+SELECT * FROM DATA_DICTIONARY.USER_BARRIERS;
+SELECT wait("foo2");
+

=== added file 'plugin/user_locks/tests/t/dangling_barrier-master.test'
--- plugin/user_locks/tests/t/dangling_barrier-master.test	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/tests/t/dangling_barrier-master.test	2010-11-17 21:25:31 +0000
@@ -0,0 +1,1 @@
+# Force a restart before and after this test.

=== added file 'plugin/user_locks/tests/t/dangling_barrier.test'
--- plugin/user_locks/tests/t/dangling_barrier.test	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/tests/t/dangling_barrier.test	2010-11-17 21:25:31 +0000
@@ -0,0 +1,5 @@
+# The goal is to create a "dangling lock". We should be cleaning up locks on
+# exit, and this will tell us (via valgrind) if that is happending.
+set @lock_name= "dangling_barrier";
+SELECT get_lock(@lock_name);
+#SELECT if ((is_used_lock(@lock_name) = CONNECTION_ID()), "YES", "NO");

=== added file 'plugin/user_locks/wait.cc'
--- plugin/user_locks/wait.cc	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/wait.cc	2010-11-17 21:25:31 +0000
@@ -0,0 +1,60 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+#include "plugin/user_locks/module.h"
+#include "plugin/user_locks/barrier_storage.h"
+
+#include <string>
+
+namespace user_locks {
+namespace barriers {
+
+int64_t Wait::val_int()
+{
+  drizzled::String *res= args[0]->val_str(&value);
+
+  if (not res)
+  {
+    null_value= true;
+    return 0;
+  }
+  null_value= false;
+
+  if (not res->length())
+    return 0;
+
+  client::Wakeup::shared_ptr barrier= Barriers::getInstance().find(Key(getSession().getSecurityContext(), res->c_str()));
+
+  if (barrier)
+  {
+    std::cerr << "Going into wait() \n";
+    barrier->wait();
+
+    return 0;
+  }
+
+  null_value= true;
+
+  return 0;
+}
+
+} /* namespace barriers */
+} /* namespace user_locks */

=== added file 'plugin/user_locks/wait.h'
--- plugin/user_locks/wait.h	1970-01-01 00:00:00 +0000
+++ plugin/user_locks/wait.h	2010-11-17 21:25:31 +0000
@@ -0,0 +1,44 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_USER_LOCKS_WAIT_H
+#define PLUGIN_USER_LOCKS_WAIT_H
+
+namespace user_locks {
+namespace barriers {
+
+class Wait : public drizzled::Item_int_func
+{
+  drizzled::String value;
+
+public:
+  Wait() :
+    drizzled::Item_int_func()
+  {}
+
+  int64_t val_int();
+  const char *func_name() const { return "wait"; }
+  bool check_argument_count(int n) { return n == 1; }
+};
+
+} /* namespace barriers */
+} /* namespace user_locks */
+
+#endif /* PLUGIN_USER_LOCKS_WAIT_H */

